using System;
using System.IO;
using System.Xml.Serialization;
using System.Reflection;
using System.Text;
using System.CodeDom.Compiler;
using System.Configuration;
using Microsoft.CSharp;

namespace Example2b.Framework {

	public class Compiler {
	
		public CompiledRuleSet CompileRuleSetFromFile(string fileName) {
			// First we should load up the source ruleset because we need that to 
			SourceRuleSet ruleset = LoadRuleSet(fileName);
			// So now we should build the assembly for this rule set
			Assembly assembly = BuildAssembly(ruleset);
			// Now we have to activate the compiled rule list object from that assembly
			// and load it then we can return the object
			Type rulelisttype = assembly.GetType(ruleset.Namespace + ".RuleSetTest");
			CompiledRuleSet result = null;
			if (rulelisttype != null) {
				result = (CompiledRuleSet) Activator.CreateInstance(rulelisttype);
				if (result != null) {
					result.Initialize();
				}
			}
			return result;
		}

		public static SourceRuleSet LoadRuleSet(string fileName) {
			XmlSerializer serializer = 
					new XmlSerializer(typeof(SourceRuleSet));
			FileStream fs = new FileStream(fileName, FileMode.Open);
			try {
				return (SourceRuleSet)serializer.Deserialize(fs);
			} 
			finally {
				fs.Close();
			}
		}

		public static Assembly BuildAssembly(SourceRuleSet ruleset) {
			// Construct a source generator
			Generator generator = new Generator();
			string src = generator.GenerateSource(ruleset);
			// Now compile it
			CompilerResults results = CompileSource(ruleset.Assemblies, src);
			// Check for errors and at the moment we just throw an exception
			if (results.Errors.HasErrors) {
				throw new Exception(String.Format("Error compiling the source: {0}", results.Errors[0].ErrorText));
			}
			return results.CompiledAssembly;
		}

		public static CompilerResults CompileSource(string assemblies, string src) {
			// Obtain access to a C# Provider
			CSharpCodeProvider provider = new CSharpCodeProvider();   
			// Then get hold of a compiler interface
			ICodeCompiler compiler = provider.CreateCompiler(); 
			// Create a parameters object
			CompilerParameters cparams	= new CompilerParameters();   
			cparams.GenerateInMemory = true;   
			cparams.GenerateExecutable = false;   

			// Split out the assemblu list
			string[] assemblyList = assemblies.Split(new Char[] {','});
			foreach (string s in assemblyList) {
				cparams.ReferencedAssemblies.Add(s);
			}

			// Now we can do the compile, get the results back into an object
			return compiler.CompileAssemblyFromSource(cparams,src);   
		}

	}
}
